@@ -47,8 +47,7 @@ add_user_columns (GtkTreeView *treeview)
 					     "pixbuf", COL_USER_FACE,
 					     NULL);
 	g_object_set (G_OBJECT (renderer),
-		      "xpad", 3,
-		      "ypad", 6,
+		      "ypad", 3,
 		      NULL);
 	/* User full name and login, on two lines */
 	renderer = gtk_cell_renderer_text_new ();
@@ -57,8 +56,9 @@ add_user_columns (GtkTreeView *treeview)
 					     "markup", COL_USER_LABEL,
 					     NULL);
 	g_object_set (G_OBJECT (renderer),
-		      "xpad", 3,
-		      "ypad", 6,
+		      "ypad", 3,
+	              "ellipsize", PANGO_ELLIPSIZE_END,
+	              "ellipsize-set", TRUE,
 		      NULL);
 
 	gtk_tree_view_insert_column (treeview, column, -1);
@@ -68,15 +68,25 @@ static gboolean
 users_model_filter (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 {
 	GstUsersTool *tool = (GstUsersTool *) data;
+	OobsUser *user;
 	gint uid;
+	gboolean show;
 
 	gtk_tree_model_get (model, iter,
 			    COL_USER_ID, &uid,
+	                    COL_USER_OBJECT, &user,
 			    -1);
-	return (tool->showall ||
-		(uid == 0 ||
-		 (uid >= tool->minimum_uid &&
-		  uid <= tool->maximum_uid)));
+
+	if (user == NULL)
+		return FALSE;
+
+	show = (tool->showall
+	        || (oobs_user_is_root (user) && tool->showroot)
+	        || (uid >= tool->minimum_uid && uid <= tool->maximum_uid));
+
+	g_object_unref (user);
+
+	return show;
 }
 
 static GtkTreeModel*
@@ -84,6 +94,7 @@ create_users_model (GstUsersTool *tool)
 {
 	GtkListStore *store;
 	GtkTreeModel *filter_model;
+	GtkTreeModel *sort_model;
 	
 	store = gtk_list_store_new (COL_USER_LAST,
 				    GDK_TYPE_PIXBUF,
@@ -95,11 +106,18 @@ create_users_model (GstUsersTool *tool)
 	                            G_TYPE_BOOLEAN,
 				    G_TYPE_OBJECT,
 				    OOBS_TYPE_LIST_ITER);
-	filter_model = gtk_tree_model_filter_new (GTK_TREE_MODEL (store), NULL);
 
+	/* Filter model */
+	filter_model = gtk_tree_model_filter_new (GTK_TREE_MODEL (store), NULL);
 	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter_model),
 						users_model_filter, tool, NULL);
-	return filter_model;
+
+	/* Sort model */
+	sort_model = gtk_tree_model_sort_new_with_model (filter_model);
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (sort_model),
+	                                      COL_USER_LOGIN, GTK_SORT_ASCENDING);
+
+	return sort_model;
 }
 
 void
@@ -121,6 +139,8 @@ create_users_table (GstUsersTool *tool)
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (users_table));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 
+	gtk_tree_view_set_search_column (GTK_TREE_VIEW (users_table), COL_USER_LOGIN);
+
 	popup = popup_menu_create (users_table, TABLE_USERS);
 	g_object_set_data_full (G_OBJECT (users_table),
 				"popup", popup,
@@ -136,17 +156,32 @@ create_users_table (GstUsersTool *tool)
 			  G_CALLBACK (on_table_popup_menu), NULL);
 }
 
-void
-users_table_set_user (OobsUser *user, OobsListIter *list_iter, GtkTreeIter *iter)
+GtkTreeModel *
+users_table_get_model ()
 {
-	GtkWidget *users_table = gst_dialog_get_widget (GST_TOOL (tool)->main_dialog, "users_table");
-	GtkTreeModel *filter_model = gtk_tree_view_get_model (GTK_TREE_VIEW (users_table));
-	GtkTreeModel *model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
+	GtkWidget *users_table;
+	GtkTreeModel *sort_model;
+	GtkTreeModel *filter_model;
+
+	users_table = gst_dialog_get_widget (GST_TOOL (tool)->main_dialog, "users_table");
+
+	sort_model = gtk_tree_view_get_model (GTK_TREE_VIEW (users_table));
+	filter_model = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (sort_model));
+
+	return gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
+}
+
+void
+users_table_set_user (OobsUser *user, GtkTreeIter *iter)
+{
+	GtkTreeModel *model;
 	GdkPixbuf *face;
 	const char *name;
 	const char *login;
 	char *label;
 
+	model = users_table_get_model ();
+
 	face = user_settings_get_user_face (user, 48);
 	name = oobs_user_get_full_name (user);
 	login = oobs_user_get_login_name (user);
@@ -160,7 +195,6 @@ users_table_set_user (OobsUser *user, OobsListIter *list_iter, GtkTreeIter *iter
 			    COL_USER_HOME, oobs_user_get_home_directory (user),
 			    COL_USER_ID, oobs_user_get_uid (user),
 			    COL_USER_OBJECT, user,
-			    COL_USER_ITER, list_iter,
 			    -1);
 	g_free (label);
 	if (face)
@@ -174,15 +208,15 @@ users_table_set_user (OobsUser *user, OobsListIter *list_iter, GtkTreeIter *iter
  * Returns: the path to the new item
  */
 GtkTreePath *
-users_table_add_user (OobsUser *user, OobsListIter *list_iter)
+users_table_add_user (OobsUser *user)
 {
-	GtkWidget *users_table = gst_dialog_get_widget (GST_TOOL (tool)->main_dialog, "users_table");
-	GtkTreeModel *filter_model = gtk_tree_view_get_model (GTK_TREE_VIEW (users_table));
-	GtkTreeModel *model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
+	GtkTreeModel *model;
 	GtkTreeIter iter;
 
+	model = users_table_get_model ();
+
 	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-	users_table_set_user (user, list_iter, &iter);
+	users_table_set_user (user, &iter);
 
 	return gtk_tree_model_get_path (model, &iter);
 }
@@ -190,13 +224,59 @@ users_table_add_user (OobsUser *user, OobsListIter *list_iter)
 void
 users_table_clear (void)
 {
-        GtkWidget *users_table = gst_dialog_get_widget (GST_TOOL (tool)->main_dialog, "users_table");
-        GtkTreeModel *filter_model = gtk_tree_view_get_model (GTK_TREE_VIEW (users_table));
-        GtkTreeModel *model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
+        GtkTreeModel *model;
+
+	model = users_table_get_model ();
 
         gtk_list_store_clear (GTK_LIST_STORE (model));
 }
 
+/*
+ * Get selected items, translating them to child GtkListStore references.
+ */
+GList*
+users_table_get_row_references ()
+{
+	GtkWidget *users_table;
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GtkTreeModel *sort_model;
+	GtkTreeModel *filter_model;
+	GtkTreePath *path, *filter_path;
+	GList *paths, *elem, *list = NULL;
+
+	users_table = gst_dialog_get_widget (GST_TOOL (tool)->main_dialog, "users_table");
+	sort_model = gtk_tree_view_get_model (GTK_TREE_VIEW (users_table));
+	filter_model = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (sort_model));
+	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (users_table));
+	paths = elem = gtk_tree_selection_get_selected_rows (selection, NULL);
+
+	if (!paths)
+		return NULL;
+
+	while (elem) {
+		filter_path = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (sort_model),
+		                                                              elem->data);
+
+		path = gtk_tree_model_filter_convert_path_to_child_path (GTK_TREE_MODEL_FILTER (filter_model),
+		                                                         filter_path);
+
+		list = g_list_prepend (list, gtk_tree_row_reference_new (model, path));
+
+		gtk_tree_path_free (path);
+		gtk_tree_path_free (filter_path);
+		elem = elem->next;
+	}
+
+	list = g_list_reverse (list);
+	g_list_foreach (paths, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (paths);
+
+	return list;
+}
+
 /*
  * Select the given path, translating it to a row in the filter model. Useful when
  * we only want to select a newly added item.
@@ -205,12 +285,19 @@ void
 users_table_select_path (GtkTreePath *path)
 {
 	GtkWidget *users_table = gst_dialog_get_widget (GST_TOOL (tool)->main_dialog, "users_table");
-	GtkTreeModel *filter_model = gtk_tree_view_get_model (GTK_TREE_VIEW (users_table));
+	GtkTreeModel *filter_model;
+	GtkTreeModel *sort_model;
 	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (users_table));
 	GtkTreePath *filter_path;
+	GtkTreePath *sort_path;
+
+	sort_model = gtk_tree_view_get_model (GTK_TREE_VIEW (users_table));
+	filter_model = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (sort_model));
 
 	filter_path = gtk_tree_model_filter_convert_child_path_to_path
 		(GTK_TREE_MODEL_FILTER (filter_model), path);
+	sort_path = gtk_tree_model_sort_convert_child_path_to_path
+		(GTK_TREE_MODEL_SORT (sort_model), filter_path);
 
 	gtk_tree_selection_unselect_all (selection);
 	gtk_tree_selection_select_path (selection, filter_path);
@@ -228,6 +315,42 @@ users_table_select_first (void)
 	gtk_tree_selection_select_path (selection, first_user);
 }
 
+void
+users_table_update_current ()
+{
+	GtkWidget *users_table = gst_dialog_get_widget (GST_TOOL (tool)->main_dialog, "users_table");
+	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (users_table));
+	GtkTreeModel *sort_model;
+	GtkTreeModel *filter_model;
+	GList *selected;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	GtkTreeIter sort_iter;
+	GtkTreeIter filter_iter;
+	OobsUser *user;
+
+	selected = gtk_tree_selection_get_selected_rows (selection, &sort_model);
+	g_assert (selected != NULL);
+
+	/* Only choose the first selected user */
+	path = (GtkTreePath *) selected->data;
+
+	gtk_tree_model_get_iter (sort_model, &sort_iter, path);
+	g_list_foreach (selected, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (selected);
+
+	gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (sort_model),
+	                                                &filter_iter, &sort_iter);
+
+	filter_model = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (sort_model));
+	gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (filter_model),
+	                                                  &iter, &filter_iter);
+
+	user = users_table_get_current ();
+	users_table_set_user (user, &iter);
+	g_object_unref (user);
+}
+
 /*
  * Convenience function to get the first selected user of the table,
  * i.e. the one whose settings are currently shown.
@@ -246,8 +369,8 @@ users_table_get_current (void)
 
 	selected = gtk_tree_selection_get_selected_rows (selection, &model);
 
-	if (selected == NULL)
-		return NULL;
+	if (!selected)
+	    return NULL;
 
 	/* Only choose the first selected user */
 	path = (GtkTreePath *) selected->data;
@@ -255,7 +378,6 @@ users_table_get_current (void)
 	gtk_tree_model_get_iter (model, &iter, path);
 	g_list_foreach (selected, (GFunc) gtk_tree_path_free, NULL);
 	g_list_free (selected);
-
 	gtk_tree_model_get (model, &iter, COL_USER_OBJECT, &user, -1);
 	return user;
 }
